""" 
 * (C) COPYRIGHT International Business Machines Corp. 2012,2013. All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
"""

##############################
# wsadmin script to deploy SSO
##############################

import java.lang.String as jstr
import java.util.Properties as jprops
import java.io as jio
import javax.management as jmgmt

########### Start Variable Definitions ###########

def getRequiredProperty(name):
	"""Get the value of a named argument.  Raise an Exception if the property is None."""
	value = getProperty(name)
	if value is None:
		raise Exception('Non-null value must be specified for argument [' + name + ']')
	return value


def getProperty(name, default=None):
	"""Get the value of a property.  If not found, return default.  If default not specified, return None."""
	if name in args.keys():
		return args[name]
	else:
		return default

# Required
wasHome = getRequiredProperty('wasHome')
profileName = getRequiredProperty('profileName')
nodeName = getRequiredProperty('nodeName')
cellName = getRequiredProperty('cellName')
serverName = getRequiredProperty('serverName')
host = getRequiredProperty('ssoHost')
port = getRequiredProperty('ssoPort')

appName = getRequiredProperty('appName')
groupName = getRequiredProperty('groupName')
ltpaPassword = getRequiredProperty('ltpaPassword')

# Optional
domainName = getProperty('domainName')
keyFile = getProperty('keyFile') 

doGenerateLtpaKey = getProperty('generateLtpaKey', 'false')

uri = getProperty('registryUri', '/emptoris/userRegistry')
realmName = getProperty('securityRealm', 'MyRealm')
registryClass = getProperty('registryImplClass', 'com.emptoris.ltpa.registry.SSMUserRegistry')
userRegistryProtocol = getProperty('registryProtocol', 'http')
secDomainDescription = getProperty('securityDomainDescription', 'Emptoris Suite')
secDomainName = getProperty('securityDomainName', 'Emptoris')


def generateLtpaKey(password):
	"""Generate an LTPA key given the specified password.  Modifies the runtime."""
	print "Generating LTPA Key"

	securityON=jmgmt.ObjectName(AdminControl.queryNames('*:*,name=SecurityAdmin'))
	AdminControl.invoke_jmx(securityON,'generateKeys', [password], ['java.lang.String'])

	print "Done generating LTPA key."
	return


def exportLtpaKey(keyFile, password):
	"""Export the LTPA key to the specified keyFile"""
	print "Exporting LTPA Key to [" + keyFile + "]"
	password=jstr(password).getBytes()
	
	securityON=jmgmt.ObjectName(AdminControl.queryNames('*:*,name=SecurityAdmin'))
	ltpaKeys=AdminControl.invoke_jmx(securityON,'exportLTPAKeys', [password], ['[B'])

	fout=jio.FileOutputStream(keyFile)
	ltpaKeys.store(fout,'')
	fout.close()

	print "Done exporting LTPA key."
	return


def importLtpaKey(keyFile, password):
	"""Import the LTPA key from the specified keyFile"""
	print "Importing LTPA Key from [" + keyFile + "]"
	password=jstr(password).getBytes()
	securityON=jmgmt.ObjectName(AdminControl.queryNames('*:*,name=SecurityAdmin'))

	fin=jio.FileInputStream(keyFile)
	ltpaKeys=jprops()
	ltpaKeys.load(fin)
	fin.close()

	AdminControl.invoke_jmx(securityON,'importLTPAKeys', [ltpaKeys, password], ['java.util.Properties', '[B'])
	
	print "Done importing LTPA key."
	return


try:
	print "Configuring WebSphere for Emptoris SSO..."
	
	# Configure SSO domain name
	if domainName is not None and not domainName == '':
		print "Domain name is " + domainName
		domainName = domainName.strip()
		print "Domain name is " + domainName
		sec = AdminConfig.getid('/Security:/')
		props = AdminConfig.showAttribute(sec, 'authMechanisms')
		props = props[props.find('[') + 1 : props.find(']')]
		props = props.split()
		for propId in props:
			authConfig = AdminConfig.showAttribute(propId, 'authConfig')
			if authConfig == 'system.LTPA':
				ssoId = AdminConfig.showAttribute(propId, 'singleSignon')
				AdminConfig.modify(ssoId, '[[domainName ' + domainName + ']]')
				# Must save here or we blow up with a security exception later.
				print "Saving domain name configuration..."
				AdminConfig.save()
				break


	# Configure security domain:
	# See if it exists.  If not, create it
	exists = 0
	for secDomain in AdminTask.listSecurityDomains().splitlines():
		if secDomain == secDomainName:
			exists = 1
			break 
	if not exists:
		AdminTask.createSecurityDomain('[-securityDomainName ' + secDomainName + ' -securityDomainDescription "' + secDomainDescription + '"]')
	
	# Server must be running to do the following.  Yuck!
	print "Configuring registry at " + userRegistryProtocol + '://' + host + ':' + str(port) + uri
	
	AdminTask.configureAppCustomUserRegistry('[-customRegClass ' + registryClass + ' -customProperties ["userRegistryUrl=' + userRegistryProtocol + '://' + host + ':' + str(port) + uri + '","jndiDataSourcePath=ssmDBnonXA","loggingLevel=Error"] -securityDomainName ' + secDomainName + ' -realmName -ignoreCase false -verifyRegistry false ]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -activeUserRegistry CustomUserRegistry]') 

	# TODO: The following is what the admin console UI generates.  Is this all necessary?
	AdminTask.modifySecurityDomain('[-securityDomainName ' + secDomainName + ' -securityDomainDescription "Emptoris Suite"]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -appSecurityEnabled true]')
	AdminTask.unsetAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -unsetEnforceJava2Security true -unsetIssuePermissionWarning true -unsetEnforceFineGrainedJCASecurity true]')
	AdminTask.configureAppWIMUserRegistry('[-securityDomainName ' + secDomainName + ' -useGlobalFederatedRepository false ]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -activeUserRegistry CustomUserRegistry]')
	AdminTask.unconfigureTrustAssociation('[-securityDomainName ' + secDomainName + ' ]')
	AdminTask.unconfigureSpnego('[-securityDomainName ' + secDomainName + ']')
	AdminTask.unconfigureCSIInbound('[-securityDomainName ' + secDomainName + ']')
	AdminTask.unconfigureCSIOutbound('[-securityDomainName ' + secDomainName + ']')
	AdminTask.unconfigureJAASLogin('[-securityDomainName ' + secDomainName + ' -loginType application ]')
	AdminTask.unconfigureJAASLogin('[-securityDomainName ' + secDomainName + ' -loginType system ]')
	AdminTask.unconfigureJaspi('[-securityDomainName ' + secDomainName + ']')
	AdminTask.unconfigureAuthzConfig('[-securityDomainName ' + secDomainName + ' ]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -customProperties ["com.ibm.security.SAF.authorization="]]')
	AdminTask.setLTPATimeout('[-securityDomainName ' + secDomainName + ' -timeout ]')
	AdminTask.unsetAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -unsetUseDomainQualifiedUserNames true -unsetCacheTimeout true]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -customProperties ["com.ibm.websphere.security.util.authCacheEnabled=","com.ibm.websphere.security.util.authCacheCustomKeySupport=","com.ibm.websphere.security.util.authCacheSize=","com.ibm.websphere.security.util.authCacheMaxSize="]]')
	AdminTask.setAppActiveSecuritySettings('[-securityDomainName ' + secDomainName + ' -customProperties ["was.security.EnableSyncToOSThread=","was.security.EnableRunAsIdentity="]]')
	
	# Map the security domain
	isMapped = 0
	for resource in AdminTask.listResourcesInSecurityDomain('-securityDomainName ' + secDomainName).splitlines():
		resource = resource.lower()
		if resource.find('cell=' + cellName.lower()) > -1 and resource.find('node=' + nodeName.lower()) and resource.find('server=' + serverName.lower()):
			isMapped = 1
			break
	if not isMapped:
		AdminTask.mapResourceToSecurityDomain('[-securityDomainName ' + secDomainName + ' -resourceName Cell=' + cellName +':Node=' + nodeName + ':Server=' + serverName + ']')

	# Usergroup mapping
	AdminApp.edit(appName, '[ -MapRolesToUsers [[ authenticated-users AppDeploymentOption.No AppDeploymentOption.No "" ' + groupName + ' AppDeploymentOption.No "" group:' + realmName + '/' + groupName + ' ]]]' ) 

	# Either generate or import a key
	if doGenerateLtpaKey.strip().lower() == 'true':
		generateLtpaKey(ltpaPassword)
	else:
		if keyFile is not None:
			importLtpaKey(keyFile, ltpaPassword)
		else:
			raise Exception('doGenerateLtpaKey is false and no keyFile is specified.  Must specify a key file if not generating an LTPA key.')

	print "Saving..."
	AdminConfig.save()
except Exception, e:
    print e
    print "SSO configuration failed.  Rolling back changes."
    AdminConfig.reset()
    raise # Re-throw to terminate the script